----------------------------------------------------------------------------------
--
-- ItemsMatrix.lua
--
--
-- This mod is a completely revamped approach to LootLink and SellValue.
-- This mod includes a inventory check window where you can view the prices of items within your inventory without
-- having to pass you mouse over them.  It stores items gathered from inspection, auction, merchant, loot, and every chat
-- channel you can think of.  If an items rarity cannot be determined, ItemsMatrix will still store the item and give it
-- an unknown khaki color.  Search and Item editing features are included as well as support for Auctioneer and ReagentInfo.
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
--GAMETOOLTIP HOOKS
local IM_lOriginal_GameTooltip_SetLootItem;
local IM_lOriginal_GameTooltip_OnHide;
local ItemsMatrix_Saved_GameTooltip_OnEvent;
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
--GAME HOOKS
local IM_lOriginal_ContainerFrameItemButton_OnEnter;
local IM_lOriginal_ContainerFrame_Update;
local IM_originalChatFrame_OnEvent;
local IM_lOriginal_SetItemRef;
local IM_lOriginal_MerchantFrame_Update;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
--LOCAL VARIABLES
local IM_Variables_Loaded = 0;			--switch to denote if variables have been indeed loaded
local IM_PlayerEnter_Loaded = 0;		--switch to denote if PlayerEnter has been indeed processed

--dropdown
local ItemsMatrix_DROPDOWN_LIST = {
	{ name = IM_SORT_NAME, sortType = "name" },
	{ name = IM_SORT_RARITY, sortType = "rarity" },
};
--chat
local ChatMessageTypes = {
	"CHAT_MSG_SYSTEM",
	"CHAT_MSG_SAY",
	"CHAT_MSG_TEXT_EMOTE",
	"CHAT_MSG_YELL",
	"CHAT_MSG_WHISPER",
	"CHAT_MSG_PARTY",
	"CHAT_MSG_GUILD",
	"CHAT_MSG_OFFICER",
	"CHAT_MSG_CHANNEL",
	"CHAT_MSG_RAID",
	"CHAT_MSG",
	"CHAT_MSG_LOOT",
};
--inventory
local ItemsMatrix_INVENTORY_SLOT_LIST = {
	{ name = "HeadSlot" },
	{ name = "NeckSlot" },
	{ name = "ShoulderSlot" },
	{ name = "BackSlot" },
	{ name = "ChestSlot" },
	{ name = "ShirtSlot" },
	{ name = "TabardSlot" },
	{ name = "WristSlot" },
	{ name = "HandsSlot" },
	{ name = "WaistSlot" },
	{ name = "LegsSlot" },
	{ name = "FeetSlot" },
	{ name = "Finger0Slot" },
	{ name = "Finger1Slot" },
	{ name = "Trinket0Slot" },
	{ name = "Trinket1Slot" },
	{ name = "MainHandSlot" },
	{ name = "SecondaryHandSlot" },
	{ name = "RangedSlot" },
};



---------------------------------------------------
-- ItemsMatrix_OnLoad
-----------------------------------------------
function ItemsMatrix_OnLoad()

	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	this:RegisterEvent("UNIT_INVENTORY_CHANGED");
	this:RegisterEvent("BANKFRAME_OPENED");
	this:RegisterEvent("AUCTION_ITEM_LIST_UPDATE");
	this:RegisterEvent("MERCHANT_SHOW");
	this:RegisterEvent("BAG_UPDATE");


	--register chat types
	for index, value in ChatMessageTypes do
		this:RegisterEvent(value);
	end

	--register slash commands
	SLASH_ITEMSMATRIX1 = "/itemsmatrix";
	SLASH_ITEMSMATRIX2 = "/im";
	SlashCmdList["ITEMSMATRIX"] = function(msg)
		ItemsMatrix_SlashCommandHandler(msg);
	end
	
	--Add the slash command to ingame repository
        SlashCmdList["ITEMSMATRIX_INVENTORYLIST"] = ItemsMatrixINV_SlashCommandHandler;
        SLASH_ITEMSMATRIX_INVENTORYLIST1 = "/iminv";
	
	--Hook the lootitem tooltip
	IM_lOriginal_GameTooltip_SetLootItem = GameTooltip.SetLootItem;
	GameTooltip.SetLootItem = ItemsMatrix_GameTooltip_SetLootItem;
	
	--Hook the gametooltip onhide
	IM_lOriginal_GameTooltip_OnHide = GameTooltip_OnHide;
	GameTooltip_OnHide = ItemsMatrix_GameTooltip_OnHide;
	
	--Hook the container frame
	IM_lOriginal_ContainerFrameItemButton_OnEnter = ContainerFrameItemButton_OnEnter;
	ContainerFrameItemButton_OnEnter = ItemsMatrix_ContainerFrameItemButton_OnEnter;
	
	--Hook the container update
	IM_lOriginal_ContainerFrame_Update = ContainerFrame_Update;
	ContainerFrame_Update = ItemsMatrix_ContainerFrame_Update;

	--Hook for the Merchant Prices
	IM_lOriginal_MerchantFrame_Update = MerchantFrame_Update;
	MerchantFrame_Update = ItemsMatrix_MerchantFrame_Update;

	--Hook the ChatFrame_OnEvent function
	IM_originalChatFrame_OnEvent = ChatFrame_OnEvent;
	ChatFrame_OnEvent = ItemsMatrix_ChatFrame_OnEvent;
	
	--Hook the SetItemRef tooltip
	IM_lOriginal_SetItemRef = SetItemRef;
	SetItemRef = ItemsMatrix_SetItemRef;


	--Display a message that ItemsMatrix is loaded
	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("|c0000FF00Derkyle's ItemsMatrix Ver. ["..ItemsMatrix_Version.."]|r");
	end
	
	
	--add our frames to the special frames, this way when a user press's escape it will
	--close the window
	tinsert(UISpecialFrames, "ItemsMatrixFrame");
	tinsert(UISpecialFrames, "ItemsMatrix_Inv_Frame");
	tinsert(UISpecialFrames, "ItemsMatrix_EditFrame");

end


---------------------------------------------------
-- ItemsMatrix_SlashCommandHandler
---------------------------------------------------
function ItemsMatrix_SlashCommandHandler(msg)

local command = string.lower(msg);


	--the stuff below is self explanatory. If you know how to code of course ;)
	if(command == "inv") then
	
		--make sure the variables are loaded
		if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then
		
			    if ItemsMatrix_Inv_Frame:IsVisible() then
				HideUIPanel(ItemsMatrix_Inv_Frame);
				PlaySound("igBackPackClose");
			    else
				ShowUIPanel(ItemsMatrix_Inv_Frame);
				PlaySound("igBackPackOpen");
			    end
		    
		elseif(IM_Database_Upgrade == 1) then

			StaticPopup_Show("IM_DATA_INCOMPATIBLE");

		end
		
	
	elseif(command == "reset") then
		
		StaticPopup_Show("IM_CLEARDATA_CONFIRM");
		
		
	elseif( command == "join") then
	
		--if(ItemsNet_JoinChannel) then
		
		--	ItemsNet_JoinChannel("IMItemsNet");
		--else
		--	DEFAULT_CHAT_FRAME:AddMessage("|c00FF0000ItemsMatrix: Error mod not loaded properly!");
		--end
		
	elseif( command == "leave") then
		
		--LeaveChannelByName("IMItemsNet", "", DEFAULT_CHAT_FRAME:GetID());
		
		--IMItemsNet_ChannelNumber = 0; --reset
		
	
	elseif(command == nil or not command or command == "") then
		
		--make sure the variables are loaded
		if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then
	
			ItemsMatrixFrame:Show();
			
		elseif(IM_Database_Upgrade == 1) then

			StaticPopup_Show("IM_DATA_INCOMPATIBLE");

		end
	
	
	end
	
	
	

end



---------------------------------------------------
-- ItemsMatrixINV_SlashCommandHandler
---------------------------------------------------
function ItemsMatrixINV_SlashCommandHandler()


	--make sure the variables are loaded
	if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then
		
	    if ItemsMatrix_Inv_Frame:IsVisible() then
		HideUIPanel(ItemsMatrix_Inv_Frame);
		PlaySound("igBackPackClose");
	    else
		ShowUIPanel(ItemsMatrix_Inv_Frame);
		PlaySound("igBackPackOpen");
	    end
	    
    
	elseif(IM_Database_Upgrade == 1) then

		StaticPopup_Show("IM_DATA_INCOMPATIBLE");

	end
    
end


---------------------------------------------------
-- IMHelp_DisplayInformation
---------------------------------------------------
function IMHelp_DisplayInformation()
local storeHelp;

	--this is a nightmare but it's either this or coding it into the xml

	storeHelp = "/im -displays IM\n/itemsmatrix -displays IM";
	storeHelp = storeHelp.."\n/im reset -resets your data to zero items.";
	storeHelp = storeHelp.."\n/iminv -displays INV_Chk";
	storeHelp = storeHelp.."\n\nTo edit an item within the ItemsMatrix window, simply right click the item for the editing GUI to appear.  Please be advised that once you delete an item it's irrevesable.  You cannot undo any items that you have deleted.  Please make sure you make a backup of your SavedVariables.lua before proceeding to alter any items.\n\nTo link an item into the chatbox from ItemsMatrix, simply Shift+LeftClick the item you wish to link.";
	storeHelp = storeHelp.."\n\nThe INV_CHK window can be used as an alternate bag.  You can use (right click) and sell (left click) items just as you normally would in your regular bags.";
	storeHelp = storeHelp.."\n\nOptions Menu:\n\nPlease becareful when using the settings located under the Options Menu.  When an option is selected to force data to a specific mod.  ItemsMatrix will send the data forcefully to process rather then the mod doing it itself.  Be warned doing this may cause information to repeat in the tooltip.";
	storeHelp = storeHelp.."\n\nThe intergrated ReagentData support (NOT ReagentInfo!) shows the exact information as ReagentInfo minus the TradeSkill Recipes.";
	storeHelp = storeHelp.."\n\n";
	storeHelp = storeHelp.."\n\n------------------------------";
	storeHelp = storeHelp.."\n\nItemsNet:";
	storeHelp = storeHelp.."\n\nItemsNet is a database sharing routine built into ItemsMatrix.  When activated you will share items with people within the same server.  Any items that are contributed by any other ItemsNet users will also be added to your list.  Please be advised that in order for you to recieve items you must have ItemsNet option enabled.  It wouldn't be fair to recieve without contributing yourself.";
	storeHelp = storeHelp.."\n\nAll items that are recieved and processed will have the tooltip tagged with the person you recieved it from and the server they are on.";
	storeHelp = storeHelp.."\n\n------------------------------";
	storeHelp = storeHelp.."\n\n";
	storeHelp = storeHelp.."\n\n------------------------------";
	storeHelp = storeHelp.."\n\nDisconnections:";
	storeHelp = storeHelp.."\n\nBlizzard has inserted an item checking system since patch 1.5.  Items you have stored in your database that you haven't seen yourself in your gaming experience will cause you to disconnect.";
	storeHelp = storeHelp.."\n\nIn short any mods that add items to your database will probally cause you disconnect.  Mods such as IM-DB and IMDB will have items that will cause you to crash as well.";
	storeHelp = storeHelp.."\n\nTo prevent future disconnections, just stick to the items you have gathered.";
	storeHelp = storeHelp.."\n\nIT IS RECOMMENDED THAT YOU DON'T USE ALREADY COMPILED DATABASES BY OTHER PLAYERS!  YOU HAVE BEEN WARNED!";
	storeHelp = storeHelp.."\n\n------------------------------";
	storeHelp = storeHelp.."\n\n";
	storeHelp = storeHelp.."\n\n------------------------------";
	storeHelp = storeHelp.."\n\nSafe Linking:";
	storeHelp = storeHelp.."\n\nItemsMatrix has a built in item link check routine.  It will alert you which items are able to be linked and which should not.  You can find the display at the bottom of each tooltip.";
	storeHelp = storeHelp.."\n\nItem is Not Linkable - This stored link was either shared via ItemsNet or was stored from a prior upgrade.  It is advisable not to link these items.  ItemsMatrix won't let you link any items that could possibly disconnect you.";
	storeHelp = storeHelp.."\n\nItem is Linkable - ItemsMatrix has determined that this item is safe for linking.";
	storeHelp = storeHelp.."\n\n------------------------------";
	storeHelp = storeHelp.."\n\n";
	storeHelp = storeHelp.."\n\n------------------------------";
	storeHelp = storeHelp.."\n\nMoney System:";
	storeHelp = storeHelp.."\n\nIn order to prevent as many errors as possible, ItemsMatrix has been changed to a text display for money.  Future requests to have the original money display will be rejected.";
	storeHelp = storeHelp.."\n\n------------------------------";
	storeHelp = storeHelp.."\n\n";
	IMHelp_FrameText:SetText(storeHelp);

end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ItemsMatrix_OnEvent
-----------------------------------------------
function ItemsMatrix_OnEvent()

	--main variables loaded so lets load everything
	if( event == "VARIABLES_LOADED" ) then

		--make sure we have a database to work with
		if( not IMDB ) then
			IMDB = { };			
		end

		--get the realm they are on
		if(GetCVar("realmName")) then
			IM_Realm = GetCVar("realmName");
		end

		--make sure we have the server listed
		if(IMDB and IM_Realm and not IMDB[IM_Realm] ) then
			IMDB[IM_Realm] = { };			
		end
		
		--create if it doesn't exist
		if(IM_Options == nil) then
			IM_Options = { };
		end

		--register the ivnentory slots
		for index = 1, getn(ItemsMatrix_INVENTORY_SLOT_LIST), 1 do
			ItemsMatrix_INVENTORY_SLOT_LIST[index].id = GetInventorySlotInfo(ItemsMatrix_INVENTORY_SLOT_LIST[index].name);
		end

		--set the variable loaded to on
		IM_Variables_Loaded = 1;
	
		--Set to Default
		GameTooltip.isDone = nil;


	--check inspecting
	elseif( event == "PLAYER_TARGET_CHANGED" ) then

		--make sure the variables are loaded
		if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then

			if( UnitIsUnit("target", "player") ) then
				return;

			--inspecting another person so check their inventory
			elseif( UnitIsPlayer("target") ) then

				ItemsMatrix_Inspect("target");

			end

		end

	--the inventory has been updated so lets check it again
	elseif( event == "UNIT_INVENTORY_CHANGED" ) then

		--make sure the variables are loaded
		if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then

			if(arg1 and arg1 == "player" ) then
				ItemsMatrix_ScanInventory(1); --we put an opt of 1 to let processlinks know not to share these items
				ItemsMatrix_Inspect("player", 1); --we put an opt of 1 to let processlinks know not to share these items
			end

			IM_InvCHK_Update(); --update the Inv-Check
		end

	--bank frame opened so lets scan that too
	elseif( event == "BANKFRAME_OPENED" ) then


		--make sure the variables are loaded
		if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then

			ItemsMatrix_ScanBank();
		end

	--scan auction window
	elseif( event == "AUCTION_ITEM_LIST_UPDATE" ) then

		--make sure the variables are loaded
		if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then

			ItemsMatrix_ScanAuction();
		end
		

	--scan merchant window
	elseif( event == "MERCHANT_SHOW" ) then
        	
		--make sure the variables are loaded
		if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then


			--this will scan the price of items
			ItemsMatrix_MoneyScan(this);
			
			--scan for vendor items
			ItemsMatrix_MerchantScan();

			IM_InvCHK_Update(); --update the Inv-Check
		end
		
        	
	--scan tooltip money
	elseif( event == "TOOLTIP_ADD_MONEY" ) then

		--make sure the variables are loaded
		if(IM_Variables_Loaded == 1 and arg2 and IM_Database_Upgrade == 0) then

			return ItemsMatrix_ToolTipMoney(arg1, arg2);
		end

        	
	--scan tooltip money
	elseif( event == "BAG_UPDATE" ) then


		--make sure the variables are loaded
		if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then

			IM_InvCHK_Update(); --update the Inv-Check
		end
		
		
	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
	    	
   				
		--get the playername and store it
		if(UnitName("player") ~= "Unknown Entity" and UnitName("player")) then

			ItemsMatrix_User = UnitName("player");

		end

		-----------------------------------------------
			--get the realm name and store it
		if(GetCVar("realmName") and not IM_Realm) then
		
			IM_Realm = GetCVar("realmName");
			
			--if the realm stored doesn't match then update it
		elseif(IM_Realm and GetCVar("realmName") and IM_Realm ~= GetCVar("realmName")) then
		
			IM_Realm = GetCVar("realmName");
		end
		

		--make sure we have the server listed
		if(IMDB and IM_Realm and not IMDB[IM_Realm] ) then
			IMDB[IM_Realm] = { };	
		end
		----------------------------------------------

		--make sure we have a user
		if(ItemsMatrix_User) then

			--create if it doesn't exist
			if(IM_Options == nil) then
				IM_Options = { };
			end


			if(not IM_Options[ItemsMatrix_User]) then

				--create the user
				IM_Options[ItemsMatrix_User] = { };

				--set defaults
				IM_Options[ItemsMatrix_User].RI1 = 0;
				IM_Options[ItemsMatrix_User].RI2 = 1;
				IM_Options[ItemsMatrix_User].AUC1 = 1;
				IM_Options[ItemsMatrix_User].ENX1 = 1;
				IM_Options[ItemsMatrix_User].IMP = 1;
				IM_Options[ItemsMatrix_User].ITEMSNET = 1;
				IM_Options[ItemsMatrix_User].VENDOR = 1;
				
				--settings
				IM_Options[ItemsMatrix_User].Setting1 = 1;

				--Version (so we can compare old versions later)
				IM_Options[ItemsMatrix_User].Version = ItemsMatrix_Version;
				
				--Database Version (so we can compare old versions later)
				IM_Options["DBVersion"] = ItemsMatrix_Database_Version;
				
				--do fiters
				IM_Options[ItemsMatrix_User].Filters = { };
				IM_Options[ItemsMatrix_User].Filters[0] = 1;
				IM_Options[ItemsMatrix_User].Filters[1] = 1;
				IM_Options[ItemsMatrix_User].Filters[2] = 1;
				IM_Options[ItemsMatrix_User].Filters[3] = 1;
				IM_Options[ItemsMatrix_User].Filters[4] = 1;
				IM_Options[ItemsMatrix_User].Filters[5] = 1;
				IM_Options[ItemsMatrix_User].Filters[99] = 1;
				

			else

				--update information if not found
				if (not IM_Options[ItemsMatrix_User].RI1) then
					IM_Options[ItemsMatrix_User].RI1 = 0;

				elseif (not IM_Options[ItemsMatrix_User].RI2) then
					IM_Options[ItemsMatrix_User].RI2 = 1;

				elseif (not IM_Options[ItemsMatrix_User].AUC1) then
					IM_Options[ItemsMatrix_User].AUC1 = 1;

				elseif (not IM_Options[ItemsMatrix_User].ENX1) then
					IM_Options[ItemsMatrix_User].ENX1 = 1;

				elseif (not IM_Options[ItemsMatrix_User].IMP) then
					IM_Options[ItemsMatrix_User].IMP = 1;

				elseif (not IM_Options[ItemsMatrix_User].VENDOR) then
					IM_Options[ItemsMatrix_User].VENDOR = 1;

				elseif (not IM_Options[ItemsMatrix_User].ITEMSNET) then
					IM_Options[ItemsMatrix_User].ITEMSNET = 1;

				elseif (not IM_Options[ItemsMatrix_User].Setting1) then
					IM_Options[ItemsMatrix_User].Setting1 = 1;

				elseif (not IM_Options[ItemsMatrix_User].Version) then
					IM_Options[ItemsMatrix_User].Version = ItemsMatrix_Version;
					
				elseif (not IM_Options["DBVersion"]) then
					
					--Display Warning
					StaticPopup_Show("IM_DATA_INCOMPATIBLE");
					
				elseif(not IM_Options[ItemsMatrix_User].Filters) then
				
					IM_Options[ItemsMatrix_User].Filters = { };
					IM_Options[ItemsMatrix_User].Filters[0] = 1;
					IM_Options[ItemsMatrix_User].Filters[1] = 1;
					IM_Options[ItemsMatrix_User].Filters[2] = 1;
					IM_Options[ItemsMatrix_User].Filters[3] = 1;
					IM_Options[ItemsMatrix_User].Filters[4] = 1;
					IM_Options[ItemsMatrix_User].Filters[5] = 1;
					IM_Options[ItemsMatrix_User].Filters[99] = 1;

				end
				
				
				


			end--if(not IM_Options[ItemsMatrix_User]) then



			--CHECK DATABASE VERSION INFO
			if(IM_Options["DBVersion"] and tonumber(IM_Options["DBVersion"]) < tonumber(ItemsMatrix_Database_Version)) then

				IM_Database_Upgrade = 1;
				
				StaticPopup_Show("IM_DATA_INCOMPATIBLE");

			end
				
				

		end--if(ItemsMatrix_User) then


		--clean up the database before they use it
		ItemsMatrix_ProcessInvalid();
		
		--set
		IM_PlayerEnter_Loaded = 1;
		
		--Set to Default
		GameTooltip.isDone = nil;

		
	--process all other chat messages and catch item links
	else
		--make sure the variables are loaded
		if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then

			ItemsMatrix_ProcessLinks(arg1, nil, nil, event);
		
		end

	end
	
	
end

--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ItemsMatrix_ProcessInvalid
---------------------------------------------------
function ItemsMatrix_ProcessInvalid()
	--this functions removes invalid items from the database.
	--Usually when someone has upgraded or when they used a compiled database.
	--Which they probally did even though I warned them in the help window.
	

end



--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------
-- ItemsMatrix_ProcessLinks
-----------------------------------------------
function ItemsMatrix_ProcessLinks(sText, sOpt, sMerchant, sEvent)
	local item;
	local name;
	local storedLink;

	--check for text
	if(not sText) then return nil; end
	
	--make sure we have a playername
	if(not ItemsMatrix_User) then return nil; end
	
	--make sure we have a database
	if(not IMDB) then return nil; end
	
	--check realm
	if(not IM_Realm) then return nil; end
	
	--make sure we have the realm stored
	if(not IMDB[IM_Realm]) then return nil; end
	
	
	--start loop and get items
	for item, name in string.gfind(sText, "|Hitem:(%d+:%d+:%d+:%d+)|h%[(.-)%]|h") do

		--remove the enchantments from the item link  which is the second number
		storedLink = string.gsub(item, "^(%d+):(%d+):(%d+):(%d+)$", "%1:0:%3:%4");

		--if item and name and it doesn't exist in the database
		if(name and storedLink) then
		
			--check to see if it's a valid item
			local nameGIF, linkGIF, qualityGIF, minLevelGIF, classGIF, subclassGIF, maxStackGIF = GetItemInfo("item:"..storedLink);



			--link and quality amd item never added
			if (nameGIF and linkGIF and qualityGIF and not IMDB[IM_Realm][nameGIF]) then

				--check filter
				if(IM_CheckRarity_Filter(qualityGIF)) then

					IM_AddData(nameGIF,"quality",qualityGIF);
					IM_AddData(nameGIF,"item",linkGIF);

					if(sMerchant) then
						IM_AddData(nameGIF,"vendor",sMerchant);
					end

					UIParent.TooltipButton = 1;
					ItemsMatrixTooltip:SetOwner(UIParent, "ANCHOR_BOTTOMRIGHT");
					ItemsMatrixTooltip:SetHyperlink(linkGIF);
					ItemsMatrix_StoreItemInfo(nameGIF);
					ItemsMatrixTooltip:Hide();

				end
				
				


			--if the item already exists then update it
			elseif(nameGIF and linkGIF and qualityGIF and IMDB[IM_Realm][nameGIF]) then

					--check quality
					if( IM_GetData(nameGIF,"quality") and tonumber(IM_GetData(nameGIF,"quality")) ~= tonumber(qualityGIF) ) then

						IM_SetData(nameGIF,"quality", qualityGIF);

					--add the quality if missing
					elseif( not IM_GetData(nameGIF,"quality") and qualityGIF) then

						IM_AddData(nameGIF,"quality", qualityGIF);
					end
					----------------------------------------------

					----------------------------------------------
					--update linkinfo
					if( IM_GetData(nameGIF,"item") and IM_GetData(nameGIF,"item") ~=  linkGIF ) then

						IM_SetData(nameGIF,"item", linkGIF);

					--add the linkinfo if missing
					elseif( not IM_GetData(nameGIF,"item") and linkGIF) then

						IM_AddData(nameGIF,"item", linkGIF);
					end
					----------------------------------------------	

					----------------------------------------------
					--update the vendor price
					if( sMerchant and IM_GetData(nameGIF,"vendor") and tonumber(IM_GetData(nameGIF,"vendor")) ~=  tonumber(sMerchant)) then

						IM_SetData(nameGIF,"vendor", sMerchant);

					--add vendor price if missing
					elseif(not IM_GetData(nameGIF,"vendor") and sMerchant) then

						IM_AddData(nameGIF,"vendor", sMerchant);

					end
					----------------------------------------------


					--update search stuff
					UIParent.TooltipButton = 1;
					ItemsMatrixTooltip:SetOwner(UIParent, "ANCHOR_BOTTOMRIGHT");
					ItemsMatrixTooltip:SetHyperlink(linkGIF);
					ItemsMatrix_StoreItemInfo(nameGIF);
					ItemsMatrixTooltip:Hide();
						
			
				
			--sometimes the item is still not classified as valid even if we
			--see it in chat.  Therefore we have to force the item to load.
			--to do this we have to set it as a tooltip
			elseif(storedLink and name) then
			
				
					
					
					
				--make sure the item isn't already added
				if(not IMDB[IM_Realm][name]) then

					--add it as a negative quality
					IM_AddData(name,"quality",-1);
					IM_AddData(name,"item","item:"..storedLink);

					if(sMerchant) then
						IM_AddData(name,"vendor",sMerchant);
					end

				end

				
				
			end--if (linkGIF and qualityGIF and not IMDB[IM_Realm][nameGIF]) then
			
			

			
			
		end--if(name and storedLink) then
		

	end--end for
	
	
	return nil;
					

end



---------------------------------------------------
-- ItemsMatrix_StoreItemInfo
---------------------------------------------------
function ItemsMatrix_StoreItemInfo(sIM_Item)
local sTipLeft;
local sTipRight;



	--set tooltip
	sTipLeft = "ItemsMatrixTooltipTextLeft";
	sTipRight = "ItemsMatrixTooltipTextRight";
	
	--avoid errors check for both tooltip objects
	if(not sTipLeft or not sTipRight) then return; end
	
	--make sure we have a playername
	if(not sIM_Item) then return nil; end
	
	--make sure we have a database
	if(not IMDB) then return nil; end
	
	--check realm
	if(not IM_Realm) then return nil; end
	
	--make sure we have the realm stored
	if(not IMDB[IM_Realm]) then return nil; end
	
		
	for index = 1, ItemsMatrixTooltip:NumLines(), 1 do
		

		field = getglobal(sTipLeft..index);
		if( field and field:IsVisible() ) then
			left = field:GetText();
		else
			left = nil;
		end
			
			
		field = getglobal(sTipRight..index);
		if( field and field:IsVisible() ) then
			right = field:GetText();
		else
			right = nil;
		end
				
		
	
		--do item locations / weapon locations
		if(left and IM_WeaponLocation[left] ~= nil) then
			
			--there are some special cases were weapontypes show on left
			--they are, crossbow, gun, thrown, wand
			--so we need to comprimise on this
			
			if(left == IM_CROSSBOW_TEXT) then
			
				--store the special weapon type
				if(not IM_GetData(sIM_Item,"wt")) then
				
					IM_AddData(sIM_Item, "wt", IM_WeaponTypes[left]);
					
				--if it doesn't match the one stored then update it
				elseif(IM_GetData(sIM_Item,"wt") and tonumber(IM_GetData(sIM_Item,"wt")) ~= IM_WeaponTypes[left]) then
				
					IM_SetData(sIM_Item,"wt",IM_WeaponTypes[left]);
				end
				
				
			elseif(left == IM_GUN_TEXT) then
			
			
				--store the special weapon type
				if(not IM_GetData(sIM_Item,"wt")) then
				
					IM_AddData(sIM_Item, "wt", IM_WeaponTypes[left]);
					
				--if it doesn't match the one stored then update it
				elseif(IM_GetData(sIM_Item,"wt") and tonumber(IM_GetData(sIM_Item,"wt")) ~= IM_WeaponTypes[left]) then
				
					IM_SetData(sIM_Item,"wt",IM_WeaponTypes[left]);
				end
				

			elseif(left == IM_THROWN_TEXT) then
			
			
				--store the special weapon type
				if(not IM_GetData(sIM_Item,"wt")) then
				
					IM_AddData(sIM_Item, "wt", IM_WeaponTypes[left]);
					
				--if it doesn't match the one stored then update it
				elseif(IM_GetData(sIM_Item,"wt") and tonumber(IM_GetData(sIM_Item,"wt")) ~= IM_WeaponTypes[left]) then
				
					IM_SetData(sIM_Item,"wt",IM_WeaponTypes[left]);
				end
				

			elseif(left == IM_WAND_TEXT) then
			
			
				--store the special weapon type
				if(not IM_GetData(sIM_Item,"wt")) then
				
					IM_AddData(sIM_Item, "wt", IM_WeaponTypes[left]);
					
				--if it doesn't match the one stored then update it
				elseif(IM_GetData(sIM_Item,"wt") and tonumber(IM_GetData(sIM_Item,"wt")) ~= IM_WeaponTypes[left]) then
				
					IM_SetData(sIM_Item,"wt",IM_WeaponTypes[left]);
				end
				
			
			else --it's not a special case
					
				--THIS IS WEAPON LOCATION NOT WEAPON TYPE!
			
				--store the special weapon type
				if(not IM_GetData(sIM_Item,"wl")) then
				
					IM_AddData(sIM_Item, "wl", IM_WeaponLocation[left]);
					
				--if it doesn't match the one stored then update it
				elseif(IM_GetData(sIM_Item,"wl") and tonumber(IM_GetData(sIM_Item,"wl")) ~= IM_WeaponLocation[left]) then
				
					IM_SetData(sIM_Item,"wl",IM_WeaponLocation[left]);
				end
			
			end

			
	
		end
	
		--do weapon types
		if(right and IM_WeaponTypes[right] ~= nil) then
			
				--store the special weapon type
				if(not IM_GetData(sIM_Item,"wt")) then
				
					IM_AddData(sIM_Item, "wt", IM_WeaponTypes[right]);
					
				--if it doesn't match the one stored then update it
				elseif(IM_GetData(sIM_Item,"wt") and tonumber(IM_GetData(sIM_Item,"wt")) ~= IM_WeaponTypes[right]) then
				
					IM_SetData(sIM_Item,"wt",IM_WeaponTypes[right]);
				end
			
		end
		
		
		--grab the tradeskill profession required for a recipe
		if(left) then
				_, _, sVariable1 = string.find(left, IM_REQUIRE_FIND);
			
				if( sVariable1 ) then
						
					_, _, sLVLVariable1 = string.find(sVariable1, IM_REQUIRE_FIND2); --check for level requirement
					
					if( sLVLVariable1 ) then
					

							--store the trade type
							if(not IM_GetData(sIM_Item,"level")) then

								IM_AddData(sIM_Item, "level", sLVLVariable1);

							--if it doesn't match the one stored then update it
							elseif(IM_GetData(sIM_Item,"level") and tonumber(IM_GetData(sIM_Item,"level")) ~= sLVLVariable1) then

								IM_SetData(sIM_Item,"level",sLVLVariable1);

							end
					
					else

						--this is for tradeskill and number required
						
						_, _, sVariable2, sVariable3 = string.find(sVariable1, IM_REQUIRE_FIND3);

						if( sVariable2 and sVariable3 and IM_TradeSkills[sVariable2] ~= nil ) then
							
							--store the trade type
							if(not IM_GetData(sIM_Item,"ts")) then

								IM_AddData(sIM_Item, "ts", IM_TradeSkills[sVariable2]);

							--if it doesn't match the one stored then update it
							elseif(IM_GetData(sIM_Item,"ts") and tonumber(IM_GetData(sIM_Item,"ts")) ~= IM_TradeSkills[sVariable2]) then

								IM_SetData(sIM_Item,"ts",IM_TradeSkills[sVariable2]);

							end

						end
					

					end--end check for level requirement
		

				end--end variable 1 check
			
						
		end			
			

		
		--do armor types
		if(right and IM_ArmorTypes[right] ~= nil) then
			
			if(not IM_GetData(sIM_Item,"at")) then
				
				IM_AddData(sIM_Item, "at", IM_ArmorTypes[right]);
					
			--if it doesn't match the one stored then update it
			elseif(IM_GetData(sIM_Item,"at") and tonumber(IM_GetData(sIM_Item,"at")) ~= IM_ArmorTypes[right]) then
				
				IM_SetData(sIM_Item,"at",IM_ArmorTypes[right]);
			end
			
		end
		
		
		--do shield types
		if(right and IM_ShieldTypes[right] ~= nil) then
			
			if(not IM_GetData(sIM_Item,"st")) then
				
				IM_AddData(sIM_Item, "st", IM_ShieldTypes[right]);
					
			--if it doesn't match the one stored then update it
			elseif(IM_GetData(sIM_Item,"st") and tonumber(IM_GetData(sIM_Item,"st")) ~= IM_ShieldTypes[right]) then
				
				IM_SetData(sIM_Item,"st",IM_ShieldTypes[right]);
			end
			
			
		end
		
		
		
	--------------------------------------------------------	
	--------------------------------------------------------		
	end --end for
	--------------------------------------------------------	
	--------------------------------------------------------


end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------
-- ItemsMatrix_OnUpdate
-----------------------------------------------
function ItemsMatrix_OnUpdate(elapsed)

	
	if(IM_PlayerEnter_Loaded == 1) then

		IM_CheckTimer = IM_CheckTimer + elapsed;
		if( IM_CheckTimer >= 0.2 ) then

			if(IM_CheckTooltip and GameTooltip:IsVisible()) then
				--check the gametooltip for any information
				
				IM_CheckTooltip = ItemsMatrix_CheckOnTooltip();
			end

			IM_CheckTimer = 0;
		end
	end

			
end


---------------------------------------------------
-- ItemsMatrix_CheckOnTooltip
-----------------------------------------------
function ItemsMatrix_CheckOnTooltip()
	
	--try not to show the money again if it's visible
	if( GameTooltip.isDone) then
		return nil;
	end

	--if not in repair mode or at a merchant or the flightpath window is opened
	--AND they aren't on the minimap then
	if(GameTooltip:IsVisible() and not InRepairMode() and not MerchantFrame:IsVisible() and not TaxiFrame:IsVisible() and ItemsMatrix_MiniMapMouseCheck() == 0) then

	
		------------------------------------------------------------
		--Frames to ignore
		if(MouseIsOver(BuffFrame)) then
			return nil;
		end
		------------------------------------------------------------

		--check to see if the first line even has information
		local sItemN = getglobal("GameTooltipTextLeft1");


		--if the line exists and it's visible then
		if( sItemN and sItemN:IsVisible() ) then

			--store the texts information
			local name = sItemN:GetText();

			--if it exists then check to see if it has an item stored
			if( name) then

				if( IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][name] ) then

					IM_UpdateOtherMods(GameTooltip, name , 1, 1);

					--it does so lets see what information we can add to tooltip
					ItemsMatrix_AddTooltipMoney(name, GameTooltip);
					
					
					--we had information so lets not send it again
					return nil;

				end
			end

		end--if( sItemN and sItemN:IsVisible() ) then
		
		
	end--major frame check

	
	--else if nothing goes through then return true
	return 1;
end


---------------------------------------------------
-- ItemsMatrix_MiniMapMouseCheck
-----------------------------------------------
function ItemsMatrix_MiniMapMouseCheck()
   local result = 0;
   
   --special thanks to the creator of TrackerAssist
   if ((MinimapCluster and MouseIsOver(MinimapCluster) and MinimapCluster:IsVisible() and
        MiniMapTrackingFrame and (not MouseIsOver(MiniMapTrackingFrame)) and
        MiniMapMailFrame and (not MouseIsOver(MiniMapMailFrame)) and
        MinimapZoomIn and (not MouseIsOver(MinimapZoomIn)) and
        MinimapZoneTextButton and (not MouseIsOver(MinimapZoneTextButton)) and
        MinimapZoomOut and (not MouseIsOver(MinimapZoomOut)) and
        GameTimeFrame and (not MouseIsOver(GameTimeFrame))) or
       (ZoomMapCluster and MouseIsOver(ZoomMapCluster) and ZoomMapCluster:IsVisible())) then
       result = 1;
   end
   
   return result;
end




--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ItemsMatrix_ChatFrame_OnEvent
-----------------------------------------------
function ItemsMatrix_ChatFrame_OnEvent(event)

	
	--check the loot msg
	if( event == "CHAT_MSG_LOOT" and arg1) then
	
		if(IM_Variables_Loaded == 1) then
		
			--this replaces the color of the picked up loot from green to it's actual color
			ItemsMatrix_ProcessLootMsg(arg1, this, ChatTypeInfo["LOOT"]);
			
			return nil;
		end
		

	
	end	


	----------------------------------------------------
	--ItemsNet
	----------------------------------------------------
	--if(IM_Options and ItemsMatrix_User and IM_Options[ItemsMatrix_User] and IM_Options[ItemsMatrix_User].ITEMSNET and IM_Options[ItemsMatrix_User].ITEMSNET == 1) then
	
		--make sure there is a message for joining and leaving
	--	if(arg1 and arg9 ) then

			----------------------------------------------------------------------------------------
	--		if(arg1 == "YOU_JOINED" and arg9 == "IMItemsNet") then

	--			DEFAULT_CHAT_FRAME:AddMessage("ItemsMatrix [ItemsNet]: Joined Channel", 156/256, 212/256, 1.0 );
				

	--			return nil;

	--		elseif(arg1 == "YOU_LEFT" and arg9 == "IMItemsNet") then

	--			DEFAULT_CHAT_FRAME:AddMessage("ItemsMatrix [ItemsNet]: Left Channel", 156/256, 212/256, 1.0 );
				
	--			
	--			return nil;
	--		end
			----------------------------------------------------------------------------------------

	--	end

		--this is to process the information sent through the ItemsNet channel
	--	if(arg1 and arg2 and arg8 and ItemsNet_GetChannelNumber("IMItemsNet") ~= 0 and ItemsNet_GetChannelNumber("IMItemsNet") == arg8) then --channel number

	--		IM_ItemsNetProcess_Chat(arg1, arg2); --send for itemsnet processing

			
	--		return nil; --do not add it to the chat! duh....

	--	end
	
	
	--elseif(arg9 == "IMItemsNet") then --otherwise ignore the channel anyways
	
	--	return nil;


	
	--end--end itemsNet check
	----------------------------------------------------
	----------------------------------------------------
	 

	IM_originalChatFrame_OnEvent(event);
end


---------------------------------------------------
-- ItemsMatrix_ProcessLootMsg
-----------------------------------------------
function ItemsMatrix_ProcessLootMsg(msg, frame, info)
local item;
local name;
local link;
local usecolor;
local storeQuality;
local storeMsg;

	--make sure we have a database
	if(not IMDB) then return nil; end
	
	--check realm
	if(not IM_Realm) then return nil; end
	
	--make sure we have the realm stored
	if(not IMDB[IM_Realm]) then return nil; end
	
	--make sure you have a msg
	if(not msg) then
		return nil;
	end
	
	--store the old message just in case
	storeMsg = msg;
	
	--now get the items
	for item, name in string.gfind(msg, "|Hitem:(%d+:%d+:%d+:%d+)|h%[(.-)%]|h") do
	
		--check
		if( item and name and IMDB[IM_Realm][name] and IM_GetData(name, "quality")) then
			
			--get the quality and convert it to a number
			storeQuality = tonumber(IM_GetData(name, "quality"));
			
			--get the color
			if(storeQuality == 5) then --legendary
				useColor = "ffff8000";
			elseif(storeQuality == 4) then --epic
				useColor = "ffa335ee";
			elseif(storeQuality == 3) then --rare
				useColor = "ff0070dd";
			elseif(storeQuality == 2) then --uncommon
				useColor = "ff1eff00";
			elseif(storeQuality == 1) then--white
				useColor = "ffffffff";
			elseif(storeQuality == 0) then--poor
				useColor = "ff9d9d9d";
			elseif(storeQuality == -1) then--poor
				useColor = "ff9d9d9d";
			else
				useColor = "ff57BDFB"; --unknown blue
			end
					
			--if color and item and name then do the link
			if(useColor and item and name) then
				--make the link
				link = "|Hitem:"..item.."|h["..name.."]|h";
				
				--subsitute the link
				msg = string.gsub(msg, IM_FixPattern(link), "|c"..useColor..link);
			end
			
		--if item and name but not in database then add it
		elseif(item and name and not IMDB[IM_Realm][name]) then
		
			ItemsMatrix_ProcessLinks(msg, nil, nil, nil);
		end
	end
	
	--make sure we have a message if we don't then do the stored one
	if(not msg and storeMsg) then
		frame:AddMessage(storeMsg, info.r, info.g, info.b, info.id);
	else
		--show the msg
		frame:AddMessage(msg, info.r, info.g, info.b, info.id);
	end
	
	
end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------




---------------------------------------------------
-- ItemsMatrix_SetItemRef
-- Chat Clickable Hyperlinks
-----------------------------------------------
function ItemsMatrix_SetItemRef( link, name, button)
	IM_lOriginal_SetItemRef( link, name, button);

	--make sure we have a link to work with
	if(not link or not name) then return nil; end
	
	--make sure we have a tooltip to work with duh
	if(not ItemRefTooltip) then return nil; end
	
	
	--we need to weed out item links and player tooltips.
	if( strsub(link, 1, 6) ~= "Player" ) then
	
	
		if( ItemRefTooltip:IsVisible() ) then
		
			local sName = ItemsMatrix_NameFromBrackets(name)
			
			if( sName and IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][sName] ) then

				--we always have to set to nil cause it won't set to nil
				--since itemref is not gametooltip_onhide
				GameTooltip.isDone = nil;
				GameTooltip.isModDone = nil;

				--Update for other mods
				IM_UpdateOtherMods(ItemRefTooltip, sName , 1);

				--add the money
				ItemsMatrix_AddTooltipMoney(sName, ItemRefTooltip, 1);



			end

		end
		
	end
	
	
end



---------------------------------------------------
-- ItemsMatrix_NameFromBrackets
-----------------------------------------------
function ItemsMatrix_NameFromBrackets(link)
	local name;
	if( not link ) then
		return nil;
	end
	for name in string.gfind(link, "|h%[(.-)%]|h|r") do
		return name;
	end
	return nil;
end




--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------



---------------------------------------------------
-- ItemsMatrix_MerchantScan
-----------------------------------------------
function ItemsMatrix_MerchantScan()
local storeCost = 0;

	if(not MerchantFrame:IsVisible()) then
		return nil;
	end
	
	local numMerchantItems = GetMerchantNumItems();
	local name, texture, price, quantity, numAvailable, isUsable;
	
	for i=1, MERCHANT_ITEMS_PER_PAGE, 1 do
		local index = (((MerchantFrame.page - 1) * MERCHANT_ITEMS_PER_PAGE) + i);

		if ( index <= numMerchantItems ) then
			name, texture, price, quantity, numAvailable, isUsable = GetMerchantItemInfo(index);
		
			--get the link for the item
			local link = GetMerchantItemLink(index)
			
			--if there is a link, quantity, and a price then set it to be stored with
			--vendor price and information
			if (link and quantity and price) then
			
				ItemsMatrix_ProcessLinks(link, nil ,quantity..":"..price, nil);
				
			--for some reason there isn't any so lets just add it anyways
			elseif (link) then
			
				ItemsMatrix_ProcessLinks(link, nil, nil, nil);
			end
		
				
		
		end
		
		
	end
	
	
end



---------------------------------------------------
-- ItemsMatrix_MerchantFrame_Update
-----------------------------------------------
function ItemsMatrix_MerchantFrame_Update()
	IM_lOriginal_MerchantFrame_Update(); --do original
	
	ItemsMatrix_MerchantScan(); --scan new page
end



---------------------------------------------------
-- ItemsMatrix_MoneyScan
-----------------------------------------------
function ItemsMatrix_MoneyScan(frame)
    frame:RegisterEvent("TOOLTIP_ADD_MONEY");
    
    local costOfItem;
    
    for bag=0,NUM_BAG_FRAMES do
      for slot=1,GetContainerNumSlots(bag) do
    
            local link = GetContainerItemLink(bag, slot);
            
            if (link and link ~= "") then
            
            
            	local name = ItemsMatrix_NameFromLink(link); --get the name
            	
            	if(name) then
            
			IM_LastPrice = 0;
			costOfItem = 0;
			ItemsMatrix_Price_Tooltip:SetBagItem(bag, slot); --set the bag slot to grab tooltip
			
			if(name and name ~= "") then

				local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot);

				
				if( itemCount and itemCount > 0 ) then --anything divided by 1 is itself
					costOfItem = floor(IM_LastPrice / itemCount);
				end
				
				
				--set the price always so that's updated
				if( name and IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][name] and costOfItem) then
				
					IM_SetData(name,"price",costOfItem);
					
				end
				
				

			end
                end

                
            end

            
            
        end
    end
    
    frame:UnregisterEvent("TOOLTIP_ADD_MONEY");
end



---------------------------------------------------
-- ItemsMatrix_Inspect
-----------------------------------------------
function ItemsMatrix_Inspect(who, sOpt)
	local index;
	
	for index = 1, getn(ItemsMatrix_INVENTORY_SLOT_LIST), 1 do
	
		local link = GetInventoryItemLink(who, ItemsMatrix_INVENTORY_SLOT_LIST[index].id);
		
		if( link ) then

			if(sOpt) then
				ItemsMatrix_ProcessLinks(link, sOpt, nil, nil);
			else
				ItemsMatrix_ProcessLinks(link, nil, nil, nil);
			end

	
		end
		
	
	
	end
end


---------------------------------------------------
-- ItemsMatrix_ScanInventory
-----------------------------------------------
function ItemsMatrix_ScanInventory(sOpt)
	local bagid;
	local size;
	local slotid;
	local link;
	
	-- NUM_BAG_FRAMES == total amount of bags possible
	
	for bagid=0,NUM_BAG_FRAMES,1 do
	
		size = GetContainerNumSlots(bagid);
		
		if( size ) then
			for slotid = size, 1, -1 do
				link = GetContainerItemLink(bagid, slotid);
				if( link ) then
				
					if(sOpt) then
						ItemsMatrix_ProcessLinks(link, sOpt, nil, nil);
					else
						ItemsMatrix_ProcessLinks(link, nil, nil, nil);
					end
				end
			end
		end
	end
end




---------------------------------------------------
-- ItemsMatrix_ScanBank
-----------------------------------------------
function ItemsMatrix_ScanBank()
	local bagid;
	local size;
	local slotid;
	local link;
	
	-- First the bank container itself
	size = GetContainerNumSlots(BANK_CONTAINER);
	for slotid = size, 1, -1 do
		link = GetContainerItemLink(BANK_CONTAINER, slotid);
		
		if( link ) then
			ItemsMatrix_ProcessLinks(link, nil, nil, nil);
		end
	end
	
	-- Now the bank bags
	-- NUM_BAG_SLOTS = 4 + 1 = 5 , NUM_BANKBAGSLOTS = 6 + 4 = 10, 1
	-- for bagid = 5, 10, 1 do
	for bagid= NUM_BAG_SLOTS + 1, (NUM_BAG_SLOTS + NUM_BANKBAGSLOTS), 1 do
	
		size = GetContainerNumSlots(bagid);
		if( size ) then
			for slotid = size, 1, -1 do
				link = GetContainerItemLink(bagid, slotid);
				
				if( link ) then
					ItemsMatrix_ProcessLinks(link, nil, nil, nil);
				end
			end
		end
	end
end



---------------------------------------------------
-- ItemsMatrix_ScanAuction
-----------------------------------------------
function ItemsMatrix_ScanAuction()
	local numBatchAuctions, totalAuctions = GetNumAuctionItems("list");
	local auctionid;
	local link;

	if( numBatchAuctions > 0 ) then
		for auctionid = 1, numBatchAuctions do
			link = GetAuctionItemLink("list", auctionid);
			if( link ) then
			
				local nameS = ItemsMatrix_ProcessLinks(link, nil, nil, nil);
				
			end
		end
	end
end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ItemsMatrix_ToolTipMoney
-----------------------------------------------
function ItemsMatrix_ToolTipMoney(frameName, money)
  -- The money in repair mode is the cost to repair, not sell
  if InRepairMode() then return; end;

    IM_LastPrice = money;   
end


---------------------------------------------------
-- GameTooltip_OnEvent
-- To Catch the Money Tooltip
-----------------------------------------------
ItemsMatrix_Saved_GameTooltip_OnEvent = GameTooltip_OnEvent;
GameTooltip_OnEvent = function ()

    if event ~= "CLEAR_TOOLTIP" then
        return ItemsMatrix_Saved_GameTooltip_OnEvent();
    elseif event == "CLEAR_TOOLTIP" then
    	IM_CheckTooltip = ItemsMatrix_CheckOnTooltip();
    end 
end


---------------------------------------------------
-- ItemsMatrix_GameTooltip_SetLootItem
-----------------------------------------------
function ItemsMatrix_GameTooltip_SetLootItem(this, slot)
	IM_lOriginal_GameTooltip_SetLootItem(this, slot);
	
	local link = GetLootSlotLink(slot);
	local name = ItemsMatrix_NameFromLink(link);
	
	if( name and IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][name] and IM_GetData(name, "price") ) then
	
		local _, _, quantity = GetLootSlotInfo(slot);
		
		IM_UpdateOtherMods(GameTooltip, name, quantity);

		ItemsMatrix_AddTooltipMoney(name, GameTooltip, quantity);
	end

end


---------------------------------------------------
-- ItemsMatrix_GameTooltip_OnHide
-----------------------------------------------
function ItemsMatrix_GameTooltip_OnHide()
	IM_lOriginal_GameTooltip_OnHide();
	
	
	--if we arent any other tooltip other then GameTooltip then ignore
	if(this:GetName() and this:GetName() == "GameTooltip") then
	
		GameTooltip.isDone = nil;
		GameTooltip.isModDone = nil;
		GameTooltip_ClearMoney();

	end
	
	
end



--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------



---------------------------------------------------
-- ItemsMatrix_AddTooltipMoney
-----------------------------------------------
function ItemsMatrix_AddTooltipMoney(name, tooltip, quantity)
local lines_added = 0;
local storeWidth= 0;
local sCheck = 0;
local storeMoney;
local storeMoneyString;
local frameStore;
local tempA = { };

	---------------------------------------
	if( not tooltip ) then
		tooltip = GameTooltip;
	end
	---------------------------------------
	
	---------------------------------------
	if( not quantity ) then
		quantity = 1;
	end
	---------------------------------------
	
	---------------------------------------
	--don't do if at merchant
	if(MerchantFrame:IsVisible()) then
		return nil;
	end
	---------------------------------------
	
	--make sure we have options
	if(not IM_Options or not ItemsMatrix_User) then return nil; end
	
	
	--make sure it's not being shown already
	if(GameTooltip.isDone) then
		return nil;
	end
	

	if(IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][name] ) then
	
	
		----------------------------------------------------
		--check to see if they want to have sell prices showing.
		if(ItemsMatrix_User and IM_Options[ItemsMatrix_User].IMP and IM_Options[ItemsMatrix_User].IMP == 1) then
		
			if(IM_GetData(name, "price")) then
				
				storeMoney = tonumber(IM_GetData(name, "price"));
				sCheck = 1;
			end
		
		--something went wrong because the setting never stored so lets show anyways
		elseif(not IM_Options[ItemsMatrix_User].IMP) then

			if(IM_GetData(name, "price")) then
			
				storeMoney = tonumber(IM_GetData(name, "price"));
				sCheck = 1;
			end
		end
		----------------------------------------------------
		
		
		--check to see if they have any vendor information
		if(IM_GetData(name, "vendor")) then
			sCheck = 1;
		end
		

		--show it if sCheck is true.
		if(sCheck == 1) then
			
				lines_added = lines_added + 1;
				--add empty string
				tooltip:AddLine(" ");
			
			
			---------------------------------------
			--show the top info line
			if( storeMoney and storeMoney ~= 0) then
				
				storeMoneyString = nil; --reset
				
				storeMoneyString = ItemsMatrix_LoadMoneyString(storeMoney, quantity );
				
				if(storeMoneyString) then
				
					lines_added = lines_added + 1;
				
					tooltip:AddDoubleLine("|c00FFFF00"..IM_SELLVALUE_COST.."[|r|c00BDFCC9"..quantity.."|r|c00FFFF00]:|r",storeMoneyString,1,1,1,1,1,1)

					frameStore = nil; --reset
					
					--return the width
					frameStore = getglobal(tooltip:GetName() .. "TextLeft" .. tooltip:NumLines());

					--return the width
					if(frameStore) then
						storeWidth = frameStore:GetWidth();
					end
		
				end
				
				
			elseif(storeMoney and storeMoney == 0) then
			
					lines_added = lines_added + 1;
					tooltip:AddLine(IM_NOSELLPRICE, 1.0, 1.0, 0);

					frameStore = nil; --reset
					
					--return the width
					frameStore = getglobal(tooltip:GetName() .. "TextLeft" .. tooltip:NumLines());

					--return the width
					if(frameStore) then
						storeWidth = frameStore:GetWidth();
					end
					
			end
			---------------------------------------
			
			

			---------------------------------------
			--Now add merchant information
			
			if(IM_GetData(name, "vendor")) then
			
				if(ItemsMatrix_User and IM_Options[ItemsMatrix_User].VENDOR and IM_Options[ItemsMatrix_User].VENDOR == 1) then
			
					tempA = { }; --reset

					tempA = IM_strsplit(":",IM_GetData(name, "vendor"));

					if(tempA and tempA[1] and tempA[2]) then


						storeMoneyString = nil; --reset

						storeMoneyString = ItemsMatrix_LoadMoneyString(tonumber(tempA[2]), 1 );

						if(storeMoneyString) then

							lines_added = lines_added + 1;

							tooltip:AddDoubleLine("|c00FFFF00"..IM_SELLVALUE_VENDORCOST.."[|r|c00BDFCC9"..tempA[1].."|r|c00FFFF00]:|r",storeMoneyString,1,1,1,1,1,1)

							frameStore = nil; --reset

							--return the width
							frameStore = getglobal(tooltip:GetName() .. "TextLeft" .. tooltip:NumLines());

							--return the width
							if(frameStore) then
								storeWidth = frameStore:GetWidth();
							end

						end


					end
				
				end
			
			end
			---------------------------------------
			
			
			
			
			--CALCULATE HEIGHT
			tooltip:SetHeight(tooltip:GetHeight() + 14*lines_added);


			---------------------------------------
			--now edit the tooltip
			if(tooltip:GetWidth() ~= nil and tooltip:GetWidth() < storeWidth) then

				tooltip:SetWidth(storeWidth); --edit the tooltip width first
				--sTooltip_ItemField:SetWidth(storeWidth);--now edit the tooltip first field

			end
			---------------------------------------
		
		
			sCheck = 0; --reset
			
		
			--for modders wanting to hook into itemsmatrix
			ItemsMatrix_AddExtraTooltipInfo(tooltip, name, quantity, name);


			--set to yes
			GameTooltip.isDone = 1; 


			tooltip:Show();
		
		
		end
			
		
		
	end--end item check
	
	
	
end




---------------------------------------------------
-- ItemsMatrix_LoadMoneyString
-----------------------------------------------
function ItemsMatrix_LoadMoneyString(sMoney, quantity)
local storeString = "";


	--don't display bogus stuff please
	if(not sMoney or sMoney == 0) then
		return nil;
	end
	
	
	--decimal check
	sMoney = floor(sMoney); --make sure it has no decimals
	
	
	--update for quantity
	if(quantity) then
		sMoney = sMoney * quantity;
	end
	
			


		--convert the long money value into seperate variables
		local gold = floor(sMoney / (COPPER_PER_SILVER * SILVER_PER_GOLD));
		local silver = floor((sMoney - (gold * COPPER_PER_SILVER * SILVER_PER_GOLD)) / COPPER_PER_SILVER);
		local copper = mod(sMoney, COPPER_PER_SILVER);

		--#FFFFFF -- white
		--#57BDFB -- light blue
		--#E2CD54 -- gold
		--#A39EA4 -- silver
		--#D7844D -- copper

		--add the gold first
		if(gold and gold ~= 0) then

			if(IM_Options and ItemsMatrix_User and IM_Options[ItemsMatrix_User] and IM_Options[ItemsMatrix_User].Setting1) then


				if(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 1) then
					storeString = "|c00FFFFFF"..gold.."|r".."|c00E2CD54g|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 2) then
					storeString = "|c00FFFFFF"..gold.."|r".." |c00E2CD54g|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 3) then
					storeString = "|c00FFFFFF"..gold.."|r".."|c00E2CD54[g]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 4) then
					storeString = "|c00FFFFFF"..gold.."|r".."|c00E2CD54[G]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 5) then
					storeString = "|c00FFFFFF"..gold.."|r".." |c00E2CD54[g]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 6) then
					storeString = "|c00FFFFFF"..gold.."|r".." |c00E2CD54[G]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 7) then
					storeString = "|c00E2CD54"..gold.."|r";
				end

			else
				storeString = "|c00FFFFFF"..gold.."|r".."|c00E2CD54[G]|r ";
			end
		end

		--add the silver next
		if(silver and silver ~= 0) then

			if(IM_Options and ItemsMatrix_User and IM_Options[ItemsMatrix_User] and IM_Options[ItemsMatrix_User].Setting1) then


				if(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 1) then
					storeString = storeString.."|c00FFFFFF"..silver.."|r".."|c00AEAEAEs|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 2) then
					storeString = storeString.. "|c00FFFFFF"..silver.."|r".." |c00AEAEAEs|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 3) then
					storeString = storeString.. "|c00FFFFFF"..silver.."|r".."|c00AEAEAE[s]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 4) then
					storeString = storeString.. "|c00FFFFFF"..silver.."|r".."|c00AEAEAE[S]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 5) then
					storeString = storeString.. "|c00FFFFFF"..silver.."|r".." |c00AEAEAE[s]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 6) then
					storeString = storeString.. "|c00FFFFFF"..silver.."|r".." |c00AEAEAE[S]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 7) then
					storeString = storeString.. "|c00FFFFFF.|r".."|c00AEAEAE"..silver.."|r";
				end

			else
				storeString = storeString.. "|c00FFFFFF"..silver.."|r".."|c00E2CD54[S]|r ";
			end

		end

		--add finally the copper
		if(copper and copper ~= 0) then

			if(IM_Options and ItemsMatrix_User and IM_Options[ItemsMatrix_User] and IM_Options[ItemsMatrix_User].Setting1) then


				if(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 1) then
					storeString = storeString.. "|c00FFFFFF"..copper.."|r".."|c00D7844Dc|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 2) then
					storeString = storeString.."|c00FFFFFF"..copper.."|r".." |c00D7844Dc|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 3) then
					storeString = storeString.. "|c00FFFFFF"..copper.."|r".."|c00D7844D[c]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 4) then
					storeString = storeString.. "|c00FFFFFF"..copper.."|r".."|c00D7844D[C]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 5) then
					storeString = storeString.. "|c00FFFFFF"..copper.."|r".." |c00D7844D[c]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 6) then
					storeString = storeString.. "|c00FFFFFF"..copper.."|r".." |c00D7844D[C]|r ";
				elseif(tonumber(IM_Options[ItemsMatrix_User].Setting1) == 7) then
					storeString = storeString.. "|c00FFFFFF.|r".."|c00D7844D"..copper.."|r";
				end

			else
				storeString = storeString.. "|c00FFFFFF"..copper.."|r".."|c00D7844D[C]|r ";
			end

		end


		if(storeString) then
			return storeString;
		else
			return nil;
		end
	

end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ItemsMatrix_BuildDisplayIndices
---------------------------------------------------
function ItemsMatrix_BuildDisplayIndices()

	if(IM_SearchActivated == 0) then
	
		local iNew = 1;
		local index;
		local value;

		IM_DisplayIndices = { };
		
		--make sure we have a database
		if(not IMDB) then return nil; end

		--check realm
		if(not IM_Realm) then return nil; end

		--make sure we have the realm stored
		if(not IMDB[IM_Realm]) then return nil; end
	
	
		--do the loop
		for index, value in IMDB[IM_Realm] do

			if(index and index ~= "" ) then

				IM_DisplayIndices[iNew] = index; --store the name of item
				iNew = iNew + 1;

			end

		end
		
		
		IM_DisplayIndices.onePastEnd = iNew;

		ItemsMatrix_Sort();

	
	end
	
	
end


---------------------------------------------------
-- ItemsMatrix_Sort
---------------------------------------------------
function ItemsMatrix_Sort()
if( ItemsMatrix_DROPDOWN_LIST[UIDropDownMenu_GetSelectedID(ItemsMatrixFrameDropDown)].sortType ) then
		local sortType = ItemsMatrix_DROPDOWN_LIST[UIDropDownMenu_GetSelectedID(ItemsMatrixFrameDropDown)].sortType;
		if( sortType == "name" ) then
			table.sort(IM_DisplayIndices);
		elseif( sortType == "rarity" ) then
			table.sort(IM_DisplayIndices, ItemsMatrix_ColorComparison);--sort by color
		end
	end

end



---------------------------------------------------
-- ItemsMatrix_ColorComparison
---------------------------------------------------
function ItemsMatrix_ColorComparison(elem1, elem2)
	local color1;
	local color2;
	
	color1 = IM_GetData(elem1, "quality");
	color2 = IM_GetData(elem2, "quality");
	
	if(color1 and color2) then
		return color2 < color1;
	else
		return nil;
	end
end


---------------------------------------------------
-- ItemsMatrix_UIDropDownMenu_SetSelectedID
---------------------------------------------------
function ItemsMatrix_UIDropDownMenu_SetSelectedID(frame, id, names)
	UIDropDownMenu_SetSelectedID(frame, id);
	if( not frame ) then
		frame = this;
	end
	UIDropDownMenu_SetText(names[id].name, frame);
end


---------------------------------------------------
-- ItemsMatrixFrameDropDown_Initialize
---------------------------------------------------
function ItemsMatrixFrameDropDown_Initialize()
	local info;
	for i = 1, getn(ItemsMatrix_DROPDOWN_LIST), 1 do
		info = { };
		info.text = ItemsMatrix_DROPDOWN_LIST[i].name;
		info.func = ItemsMatrixFrameDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end


---------------------------------------------------
-- ItemsMatrixFrameDropDown_OnLoad
---------------------------------------------------
function ItemsMatrixFrameDropDown_OnLoad()
	UIDropDownMenu_Initialize(ItemsMatrixFrameDropDown, ItemsMatrixFrameDropDown_Initialize);
	ItemsMatrix_UIDropDownMenu_SetSelectedID(ItemsMatrixFrameDropDown, 1, ItemsMatrix_DROPDOWN_LIST);
	UIDropDownMenu_SetWidth(80);
	UIDropDownMenu_SetButtonWidth(24);
	UIDropDownMenu_JustifyText("LEFT", ItemsMatrixFrameDropDown)
end


---------------------------------------------------
-- ItemsMatrixFrameDropDown_OnClick
---------------------------------------------------
function ItemsMatrixFrameDropDown_OnClick()
	local oldID = UIDropDownMenu_GetSelectedID(ItemsMatrixFrameDropDown);
	UIDropDownMenu_SetSelectedID(ItemsMatrixFrameDropDown, this:GetID());
	if( oldID ~= this:GetID() ) then
		ItemsMatrix_Refresh();
	end
end



---------------------------------------------------
-- ItemsMatrix_UpdateView
---------------------------------------------------
function ItemsMatrix_UpdateView()
local iItem;
local storeColor;

	
	if( IM_DisplayIndices == nil or IM_DisplayIndices.onePastEnd == nil ) then
		ItemsMatrix_BuildDisplayIndices();
	end

	ItemsMatrixTitleText:SetText(IM_TITLE_TEXT..IM_ReturnCount());
	
	
	if(IM_Realm) then
		ItemsMatrixVersionLabel:SetText("["..IM_Realm.."]\nVersion: "..ItemsMatrix_Version);
	else
		ItemsMatrixVersionLabel:SetText("Version: "..ItemsMatrix_Version);
	end

	--if this doesn't exist chances are they purged or something
	if(not IM_DisplayIndices.onePastEnd) then return nil; end

	--to prevent errors 5th arguement must be nil
	FauxScrollFrame_Update(ItemsMatrixListScrollFrame, IM_DisplayIndices.onePastEnd - 1, ITEMMATIX_ITEM_SHOWN, ITEMMATIX_ITEM_HEIGHT, nil);
	for iItem = 1, ITEMMATIX_ITEM_SHOWN, 1 do
		
		local itemIndex = iItem + FauxScrollFrame_GetOffset(ItemsMatrixListScrollFrame);
		local IMItemObj = getglobal("ItemsMatrixItem"..iItem);
		
		if(IMItemObj) then
		
			if( itemIndex < IM_DisplayIndices.onePastEnd ) then

				local color;
				local name = IM_DisplayIndices[itemIndex];

				--check for empty null spaces and such
				if(name) then
					
					color = ITEM_QUALITY_COLORS[tonumber(IM_GetData(name, "quality"))];

					IMItemObj:SetText(name);

					if( color) then
						IMItemObj:SetTextColor(color.r, color.g, color.b);
						IMItemObj.r = color.r;
						IMItemObj.g = color.g;
						IMItemObj.b = color.b;
					else
						IMItemObj.r = 0;
						IMItemObj.g = 0;
						IMItemObj.b = 0;
					end
					IMItemObj:Show();

				end--if(name) then

			else
				IMItemObj:Hide();
				
			end--if( itemIndex < IM_DisplayIndices.onePastEnd ) then
			
			
		end--if(IMItemObj) then
		
	end



end


---------------------------------------------------
-- ItemsMatrix_Refresh
---------------------------------------------------
function ItemsMatrix_Refresh()

	FauxScrollFrame_SetOffset(ItemsMatrixListScrollFrame, 0);
	getglobal("ItemsMatrixListScrollFrameScrollBar"):SetValue(0);
	ItemsMatrix_BuildDisplayIndices();
	ItemsMatrix_UpdateView();

end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ItemsMatrixFrame_OnShow
---------------------------------------------------
function ItemsMatrixFrame_OnShow()
	--refresh for kicks
	ItemsMatrix_Refresh();
end


---------------------------------------------------
-- ItemsMatrixFrame_OnHide
---------------------------------------------------
function ItemsMatrixFrame_OnHide()
	if( ItemsMatrixFrame.TooltipButton ) then
		ItemsMatrixFrame.TooltipButton = nil;
		HideUIPanel(GameTooltip);
	end

	if ( GameTooltip:IsVisible() ) then 
		HideUIPanel(GameTooltip);
	end


	--heck if all else fails hide it anyways
	HideUIPanel(GameTooltip);
	
	PlaySound("igMainMenuOption"); 

end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ItemsMatrixItemButton_OnEnter
---------------------------------------------------
function ItemsMatrixItemButton_OnEnter()

	if(this:GetText()) then
		
		--check for database stuff
		if(IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][this:GetText()]) then
	
			--only input if link is linkable
			if(IM_GetData(this:GetText(), "item")) then
			

				--update information if nessasary
				--check to see if it's a valid item
				local nameGIF, linkGIF, qualityGIF, minLevelGIF, classGIF, subclassGIF, maxStackGIF = GetItemInfo(IM_GetData(this:GetText(), "item"));


				if(nameGIF and linkGIF and qualityGIF) then

					--check filter
					if(not IM_CheckRarity_Filter(qualityGIF)) then
						
						IMDB[IM_Realm][this:GetText()] = nil; --then delete it cause we don't want this item
					
						--show the tooltip
						IM_ShowItem(this:GetText(), this, ItemsMatrixFrame)
						
						return nil;

					end


					--check quality if it's set to unknown then update it
					if( IM_GetData(nameGIF,"quality") and tonumber(IM_GetData(nameGIF,"quality")) == -1 and qualityGIF ) then

						IM_SetData(nameGIF,"quality", qualityGIF);

					--add the quality if missing
					elseif( not IM_GetData(nameGIF,"quality") and qualityGIF) then

						IM_AddData(nameGIF,"quality", qualityGIF);
					end
					----------------------------------------------
					
					
					
				
					UIParent.TooltipButton = 1;
					ItemsMatrixTooltip:SetOwner(UIParent, "ANCHOR_BOTTOMRIGHT");
					ItemsMatrixTooltip:SetHyperlink(IM_GetData(this:GetText(), "item"));
					ItemsMatrix_StoreItemInfo(this:GetText());
					ItemsMatrixTooltip:Hide();
					
				
				
				
				end



			end--check for item link
			
			

		end--check for database
		
		
		
		---------------------------------------
		--show the tooltip
		IM_ShowItem(this:GetText(), this, ItemsMatrixFrame)
		---------------------------------------
			
		return nil;
	
	end--check for string
	
	
	
end


---------------------------------------------------
-- ItemsMatrixItemButton_OnLeave
---------------------------------------------------
function ItemsMatrixItemButton_OnLeave()
	if( ItemsMatrixFrame.TooltipButton ) then
		ItemsMatrixFrame.TooltipButton = nil;
		HideUIPanel(GameTooltip);
	end
end


---------------------------------------------------
-- ItemsMatrixItemButton_OnClick
---------------------------------------------------
function ItemsMatrixItemButton_OnClick(button)
local sLink;
local useColor;


	if( button == "LeftButton" ) then
	
		if( IsShiftKeyDown() and ChatFrameEditBox:IsVisible() ) then
		

				--only input if link is linkable
				if(IM_GetData(this:GetText(), "item")) then


					local nameGIF, linkGIF, qualityGIF, minLevelGIF, classGIF, subclassGIF, maxStackGIF = GetItemInfo(IM_GetData(this:GetText(), "item"));

					if(linkGIF and qualityGIF and nameGIF) then

					
						if(qualityGIF == 5) then --legendary
							useColor = "ffff8000";
						elseif(qualityGIF == 4) then --epic
							useColor = "ffa335ee";
						elseif(qualityGIF == 3) then --rare
							useColor = "ff0070dd";
						elseif(qualityGIF == 2) then --uncommon
							useColor = "ff1eff00";
						elseif(qualityGIF == 1) then--white
							useColor = "ffffffff";
						elseif(qualityGIF == 0) then--poor
							useColor = "ff9d9d9d";
						elseif(qualityGIF == -1) then--poor
							useColor = "ff9d9d9d";
						else
							useColor = "ff57BDFB";
						end

						--now insert it
						if(useColor) then	
							ChatFrameEditBox:Insert("|c"..useColor.."|H"..linkGIF.."|h["..nameGIF.."]|h|r");
						end
			
					end



				end
		
		
		end
		
	elseif (button == "RightButton") then
		
		ItemsMatrix_EditFrame:Hide();
		ItemsMatrix_EditFrame_DeleteItemConfirm:Hide();
		ItemsMatrix_EditFrameEditNameText:SetText("");
	
		if(IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][this:GetText()]) then
		
			ItemsMatrix_EditFrameEditNameText:SetText(this:GetText());


			if(IM_GetData(this:GetText(), "quality")) then
			
				color = ITEM_QUALITY_COLORS[tonumber(IM_GetData(this:GetText(), "quality"))];

				if( color) then
					ItemsMatrix_EditFrameEditNameText:SetTextColor(color.r, color.g, color.b);
					ItemsMatrix_EditFrameEditNameText.r = color.r;
					ItemsMatrix_EditFrameEditNameText.g = color.g;
					ItemsMatrix_EditFrameEditNameText.b = color.b;
				end
				
			end

			--set the item rarity for the drop down
			ItemsMatrix_EditFrame:Show();
		
		end
	end
end


---------------------------------------------------
-- ItemsMatrix_DeleteItem
---------------------------------------------------
function ItemsMatrix_DeleteItem()
local getItem;

	if(not ItemsMatrix_EditFrameEditNameText) then
		return nil;
	end
	
	getItem = ItemsMatrix_EditFrameEditNameText:GetText();
	
	
	if(getItem) then
	
		if(IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][getItem] ) then
		
			--delete the item
			IMDB[IM_Realm][getItem] = nil;
			
			ItemsMatrix_EditFrameEditNameText:SetText("");
			
			--refresh the display
			ItemsMatrix_Refresh();
			
		end
	
	end
	

end



--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ItemsMatrix_ContainerFrameItemButton_OnEnter
-----------------------------------------------
function ItemsMatrix_ContainerFrameItemButton_OnEnter()
	IM_lOriginal_ContainerFrameItemButton_OnEnter();
	
local name;

	if( not InRepairMode() and not MerchantFrame:IsVisible() ) then
		local frameID = this:GetParent():GetID();
		local buttonID = this:GetID();
		local link = GetContainerItemLink(frameID, buttonID);

		if(link) then
			name = ItemsMatrix_NameFromLink(link);
		end
				
		
		if( name and IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][name]) then
		
			local _, itemCount = GetContainerItemInfo(frameID, buttonID);


				if(name and itemCount) then

					IM_UpdateOtherMods(GameTooltip, name, itemCount);

					ItemsMatrix_AddTooltipMoney(name, GameTooltip, itemCount);

				elseif(name) then

					IM_UpdateOtherMods(GameTooltip, name, 1);

					ItemsMatrix_AddTooltipMoney(name, GameTooltip, 1);
				end

			
		end
	end

end



---------------------------------------------------
-- ItemsMatrix_ContainerFrame_Update
-----------------------------------------------
--IF THIS FUNCTION IS WRONG IN ANYWAYS
--IT WILL POP UP A MESSAGE STATING ("Attempt to index a nil") it won't tell you
--what line or anything.  I pulled my hair with this one.

function ItemsMatrix_ContainerFrame_Update(frame)
	IM_lOriginal_ContainerFrame_Update(frame);
	
local name;


	if( not InRepairMode() and not MerchantFrame:IsVisible()) then
		local frameID = frame:GetID();
		local frameName = frame:GetName();
		local iButton;
		for iButton = 1, frame.size do
			local button = getglobal(frameName.."Item"..iButton);
			if( GameTooltip:IsOwned(button) ) then
			
				local buttonID = button:GetID();
				local link = GetContainerItemLink(frameID, buttonID);
				
				if(link) then
					name = ItemsMatrix_NameFromLink(link);
				end
				
				if( name and IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][name] ) then
				
					local _, itemCount = GetContainerItemInfo(frameID, buttonID);
					
					--Update for other mods
					
					if(name and itemCount) then
					
						IM_UpdateOtherMods(GameTooltip, name, itemCount);
					
						ItemsMatrix_AddTooltipMoney(name, GameTooltip, itemCount);
					
					elseif(name) then
					
						IM_UpdateOtherMods(GameTooltip, name, 1);
					
						ItemsMatrix_AddTooltipMoney(name, GameTooltip, 1);
					end
						
					
				end
			end
		end
	end

end






--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ItemsMatrix_GetHyperlink
-----------------------------------------------
function ItemsMatrix_GetHyperlink(name)

	--make sure we have a database
	if(not IMDB) then return nil; end

	--check realm
	if(not IM_Realm) then return nil; end

	--make sure we have the realm stored
	if(not IMDB[IM_Realm]) then return nil; end

	
	local itemLink = IM_GetData(name, "item");
	
	if( itemLink) then
		return itemLink;
	end
	
	
	return nil;
	
	
end


---------------------------------------------------
-- ItemsMatrix_NameFromLink
-----------------------------------------------
function ItemsMatrix_NameFromLink(link)
	local name;
	if( not link ) then
		return nil;
	end
	for name in string.gfind(link, "|c%x+|Hitem:%d+:%d+:%d+:%d+|h%[(.-)%]|h|r") do
		return name;
	end
	return nil;
end



---------------------------------------------------
-- IM_ReturnCount
-----------------------------------------------
function IM_ReturnCount()

	local IMDB_Size = 0;
	
	--make sure we have a database
	if(not IMDB) then return 0; end

	--check realm
	if(not IM_Realm) then return 0; end

	--make sure we have the realm stored
	if(not IMDB[IM_Realm]) then return 0; end
	
	for index, value in IMDB[IM_Realm] do
			
		--check for empty spaces or null and such
		if(index ~= nil and index ~= "") then
			IMDB_Size = IMDB_Size + 1;
		end
			
	end
			
	return IMDB_Size;
end


---------------------------------------------------
-- ItemsMatrix_Keypress
---------------------------------------------------
function ItemsMatrix_Keypress()


	--make sure the variables are loaded
	if(IM_Variables_Loaded == 1 and IM_Database_Upgrade == 0) then
	

		if ( ItemsMatrixFrame:IsVisible() ) then 
			ItemsMatrixFrame:Hide();
		else
			ItemsMatrixFrame:Show();
		end
	
	
	elseif(IM_Database_Upgrade == 1) then
	
		StaticPopup_Show("IM_DATA_INCOMPATIBLE");
	
	end
	
end


---------------------------------------------------
-- ItemsMatrix_GetLinkPatternCount
-----------------------------------------------
function ItemsMatrix_GetLinkPatternCount(sMsg)

   if(not sMsg) then
      return;
   end

   local i = 0;
   local iCount = 0;
   
   while true do
   
         i = string.find(sMsg, ":", i+1)    -- find 'next' ":"
         if i == nil then break end
        iCount = iCount + 1;
   
   end
   
   return iCount;

end


---------------------------------------------------
-- IM_FixPattern
-----------------------------------------------
function IM_FixPattern(string)
local result = "";
local remain = string;
local char;

local IM_Chars = { };
IM_Chars["^"] = 1;
IM_Chars["$"] = 1;
IM_Chars["("] = 1;
IM_Chars[")"] = 1;
IM_Chars["%"] = 1;
IM_Chars["."] = 1;
IM_Chars["["] = 1;
IM_Chars["]"] = 1;
IM_Chars["*"] = 1;
IM_Chars["+"] = 1;
IM_Chars["-"] = 1;
IM_Chars["?"] = 1;

	while( remain ~= "" ) do
		char = strsub(remain, 1, 1);
		if( IM_Chars[char] ) then
			result = result.."%"..char;
		else
			result = result..char;
		end
		remain = strsub(remain, 2);
	end
	
	return result;
end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- IM_UpdateOtherMods
-- This function pass's information to other mods so that they will work if installed
-----------------------------------------------
function IM_UpdateOtherMods(sTooltip, itemN, itemCount)

		--check for options
		if(ItemsMatrix_User == nil) then
			return nil;
		end

		if(IM_Options == nil) then
			return nil;
		end


		if(IM_Options[ItemsMatrix_User] == nil) then
			return nil;
		end
		
		
		--do not duplicate information
		if(GameTooltip.isModDone and GameTooltip.isModDone == 1) then
			return nil;
		end
		
		--set tooltip
		GameTooltip.isModDone = 1;
			
		
		--------------------------------------------------------------------------------

			--auctioneer
			if(IM_Options[ItemsMatrix_User].AUC1 and IM_Options[ItemsMatrix_User].AUC1 == 1) then
			
				
				if(Auctioneer_AddTooltipInfo and IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][itemN]) then

					Auctioneer_AddTooltipInfo(sTooltip, itemN , itemCount, IMDB[IM_Realm][itemN]);
				end
			
			end
			
			
			--enchantrix
			if(IM_Options[ItemsMatrix_User].ENX1 and IM_Options[ItemsMatrix_User].ENX1 == 1) then
			
				if(Enchantrix_AddTooltipInfo and IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][itemN]) then

					Enchantrix_AddTooltipInfo(sTooltip,itemN , itemCount, IMDB[IM_Realm][itemN]);
				end
				
			end
			
			
			--regeantinfo (force data to GDI Version)
			if(IM_Options[ItemsMatrix_User].RI2 and IM_Options[ItemsMatrix_User].RI2 == 1) then
			
				if(ReagentInfo_AddTooltipInfo and IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][itemN]) then

					ReagentInfo_AddTooltipInfo(sTooltip, itemN);

				end
				
			end
			


			-- pass to auctionmatrix
			if(AuctionMatrix_AddTooltipMoney and IMDB and IM_Realm and IMDB[IM_Realm] and IMDB[IM_Realm][itemN]) then
			
				if(not itemCount) then
					AuctionMatrix_AddTooltipMoney(itemN, GameTooltip, 1);
				else
					AuctionMatrix_AddTooltipMoney(itemN, GameTooltip, itemCount);
				end


			end

			
		--------------------------------------------------------------------------------
			
		
		-----------------------
		----- THE CODE BELOW WAS USED FROM REAGENTINFO WITH PERMISSION FROM GDI.
		----- SPECIAL THANKS TO GDI FOR MAKING SUCH A WONDERFUL MOD.
		-----------------------
		
		--Check wether to show built in ReagentData info
		if(IM_Options[ItemsMatrix_User].RI1 and IM_Options[ItemsMatrix_User].RI1 == 1) then
			
			--check to see if reagent data is installed
			if(ReagentData) then
			
			------------------------------------------------------------
					  -- First ask Reagent Data which professions use the specified item.
					local professions = ReagentData_GetProfessions(itemN);


					if(professions) then

						  -- Next load our colors from the player's data
						  local red = 1.0;
						  local green = 0.82;
						  local blue = 0.0;

						  -- Sort the resultant profession list alphabetically.
						  table.sort(professions);

						  -- text will hold the text to be written to the frame.
						  local text = "";

						  -- count is used to keep track of how many professions have been
						  -- displayed on a line.  My experience has been that showing more than
						  -- three professions on a single line makes the tooltip way too large,
						  -- so I force a linewrap every three.

						  local count = 0;

						  -- Loop through the professions
						  for key, value in professions do
							    text = text .. value .. ", ";
							    count = count + 1;
							    if (count == 3) then
								 text = text .. "\n";
								 count = 0;
							    end
						  end

						  -- Remove the last comma space from the text string.
						  text = gsub(text, "(.+), $", "%1");

						  -- Add in the Professions prefix
						  if (text ~= "" and text ~= nil) then
						       text = IM_REAGENTINFO_PROFESSIONS .. ": " .. text;
						  end

						  -- Finally, update the frame
						  sTooltip:AddLine(text, red, green, blue);
					  
					end
			------------------------------------------------------------



				     -- Gather skills handling code
					  -- First ask Reagent Data which skills gather the specified item.
					  local gathering = ReagentData_GatheredBy(itemN);

					  if(gathering) then
					  
						  -- Alphabetically sort the results
						  table.sort(gathering);

						  -- text will hold the string we add to the tooltip
						  local text = "";

						  -- Load the player's colors
						  local red = 0.0;
						  local green = 0.75;
						  local blue = 0.8;


						  -- Now loop through the results
						  for key, value in gathering do

							    text = text .. value .. ", ";
						  end

						  -- Remove the last comma space from the text string
						  text = gsub(text, "(.+), $", "%1");

						  -- Add the Gathered By prefix
						  if (text ~= "" and text ~= nil) then
						       text = IM_REAGENTINFO_GATHEREDBY .. ": " .. text;
						  end

						  -- Finally, add the text to the tooltip.
						  sTooltip:AddLine(text, red, green, blue);
					  
					  end

			------------------------------------------------------------
			
			
				-- Spell Reagent handling code
					-- text will hold the string to add to the tooltip.
					-- And, in case you're wondering, I have no idea
					-- why I wrote the code in three different orders in the
					-- three different sections.  I blame my cat.
					local text = "";

					-- Load the player's color choices
					local red = 1.0;
					local green = 0.0;
					local blue = 1.0;

					-- Ask Reagent Data what classes use the item as a spell reagent
					local classes = ReagentData_ClassSpellReagent(itemN);

					if(classes) then
					
						  table.sort(classes);

						  for num, class in classes do
						       text = text .. class .. ", ";
						  end

						  -- Remove the last comma space from the text string
						  text = gsub(text, "(.+), $", "%1");

						  -- Add in the Spell Reagent prefix
						  if (text ~= "" and text ~= nil) then
						       text = IM_REAGENTINFO_SPELLREAGENT .. ": " .. text;
						  end

						  -- Finally, add the text to the tooltip
						  sTooltip:AddLine(text, red, green, blue);
					  
					end

			------------------------------------------------------------

			
			end
		
		
		
		end
		
		
end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


--------------------------------------------------------------------------------------------------
-- Hookable callback functions
--------------------------------------------------------------------------------------------------
--this hook is exactly like the one used for lootlink on purpose.
--this will allow mods that already have the functionality for lootlink to work with itemsmatrix
function ItemsMatrix_AddExtraTooltipInfo(tooltip, name, quantity, item)

end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------



--THE STUFF HERE IS EXTREMELY IMPORTANT FOR THE FUNCTIONALITY OF ITEMSMATRIX
--DO NOT ALTER OR TAMPER THESE FUNCTIONS.
--IF YOU DO YOU DO AT YOUR OWN RISK!


---------------------------------------------------
-- IM_GetData
-- Returns the parsed information from the Database.
-----------------------------------------------
function IM_GetData(sName, SDelimiter)
local sText;
local s1, s2, gValue;
local sSlot;

	--for some reason the byte num from sOpt always comes out 194
	--so to compensate for this we have to assign the ascii byte ourselves
	--""= 187; -- item
	--""= 186; -- quality
	--""= 247; -- price
	--""= 162; -- wl
	--""= 163; -- wt
	--""= 167; -- ts
	--""= 183; -- at
	--""= 165; -- st
	--""= 169; -- realm
	--""= 248; -- level
	--""= 215; -- player
	--""= 177; -- vendor
	
	
	--make sure we have a item name
	if(not sName) then return nil; end

	--make sure the request being passed is a string
	if(type(sName) ~= "string") then return nil; end
	
	--make sure we have an SDelimiter
	if(not SDelimiter) then return nil; end
	
	--make sure the SDelimiter being passed is a string
	if(type(SDelimiter) ~= "string") then return nil; end
	
	--make sure we have a database to work with
	if(not IMDB) then return nil; end
	
	--make sure we have a realm name
	if(not IM_Realm) then return nil; end
	
	--make sure we have the database with the server
	if(not IMDB[IM_Realm]) then return nil; end
	
	--if it exists and it's on the realm then
	if(not IMDB[IM_Realm][sName]) then
		return nil;	
	else
		sText = IMDB[IM_Realm][sName];
	end
	
	--make sure we have an sText
	if(not sText) then return nil; end
	
	--convert to lowercase just in case
	SDelimiter = string.lower(SDelimiter);
	
	--now get the appropriate character
	if(SDelimiter == "item") then sSlot = "";
	elseif(SDelimiter == "quality") then sSlot = "";
	elseif(SDelimiter == "price") then sSlot = "";
	elseif(SDelimiter == "wl") then sSlot = "";
	elseif(SDelimiter == "wt") then sSlot = "";
	elseif(SDelimiter == "ts") then sSlot = "";
	elseif(SDelimiter == "at") then sSlot = "";
	elseif(SDelimiter == "realm") then sSlot = "";
	elseif(SDelimiter == "level") then sSlot = "";
	elseif(SDelimiter == "player") then sSlot = "";
	elseif(SDelimiter == "vendor") then sSlot = "";
	elseif(SDelimiter == "st") then sSlot = ""; end
	
	--make sure we have a slot
	if(not sSlot) then
		return nil;
	end

	--search the text
	s1, s2, gValue = string.find(sText, sSlot.."(.-)")


	--if there is a value then return it, otherwise return nil
	if( gValue ) then
		return gValue;
		
	else
		return nil;
	end
	
	
end



---------------------------------------------------
-- IM_SetData
-----------------------------------------------
function IM_SetData(sName, SDelimiter, sValue)

local sText;
local sSlots = { "", "","","","","","","","","","","" };
local sSlotNum;
local sStringChk;
local sStoreStr;


	--for some reason the byte num from SDelimiter always comes out 194
	--so to compensate for this we have to assign the ascii byte ourselves
	--""= 187; -- item
	--""= 186; -- quality
	--""= 247; -- price
	--""= 162; -- wl
	--""= 163; -- wt
	--""= 167; -- ts
	--""= 183; -- at
	--""= 165; -- st
	--""= 169; -- realm
	--""= 248; -- level
	--""= 215; -- player
	--""= 177; -- vendor
	
	--make sure we have a item name
	if(not sName) then return nil; end

	--make sure the request being passed is a string
	if(type(sName) ~= "string") then return nil; end
	
	--make sure we have an SDelimiter
	if(not SDelimiter) then return nil; end
	
	--make sure the SDelimiter being passed is a string
	if(type(SDelimiter) ~= "string") then return nil; end
	
	--make sure we have an sValue
	if(not sValue) then return nil; end
	
	--make sure we have a database to work with
	if(not IMDB) then return nil; end
	
	--make sure we have a realm name
	if(not IM_Realm) then return nil; end
	
	--make sure we have the database with the server
	if(not IMDB[IM_Realm]) then return nil; end
	
	--if it exists and it's on the realm then
	if(not IMDB[IM_Realm][sName]) then
		return nil;	
	else
		sText = IMDB[IM_Realm][sName];
	end
	
	
	--make sure we have an sText
	if(not sText) then return nil; end
	
	--convert to lowercase just in case
	SDelimiter = string.lower(SDelimiter);
	
	if(SDelimiter == "item") then sSlotNum = 187;
	elseif(SDelimiter == "quality") then sSlotNum = 186;
	elseif(SDelimiter == "price") then sSlotNum = 247;
	elseif(SDelimiter == "wl") then sSlotNum = 162;
	elseif(SDelimiter == "wt") then sSlotNum = 163;
	elseif(SDelimiter == "ts") then sSlotNum = 167;
	elseif(SDelimiter == "at") then sSlotNum = 183;
	elseif(SDelimiter == "realm") then sSlotNum = 169;
	elseif(SDelimiter == "level") then sSlotNum = 248;
	elseif(SDelimiter == "player") then sSlotNum = 215;
	elseif(SDelimiter == "vendor") then sSlotNum = 177;
	elseif(SDelimiter == "st") then sSlotNum = 165; end

	--make sure we have information to continue
	if(not sText or not sSlotNum or not sName or not sValue) then
		return nil;
	end



	sStoreStr = ""; --reset
	
	for index, value in sSlots do
	
		--if the ascii value of value doesn't equal our stored one then store
		if(string.byte(value) ~= sSlotNum) then
		
			
			--convert to grab the data
			if(string.byte(value) == 187) then
			
				sStringChk = IM_GetData(sName,"item");
				
			elseif(string.byte(value) == 186) then
			
				sStringChk = IM_GetData(sName,"quality");
				
			elseif(string.byte(value) == 247) then
			
				sStringChk = IM_GetData(sName,"price");
				
			elseif(string.byte(value) == 162) then
			
				sStringChk = IM_GetData(sName,"wl");
				
			elseif(string.byte(value) == 163) then
			
				sStringChk = IM_GetData(sName,"wt");
				
			elseif(string.byte(value) == 167) then
			
				sStringChk = IM_GetData(sName,"ts");
				
			elseif(string.byte(value) == 183) then
			
				sStringChk = IM_GetData(sName,"at");
				
			elseif(string.byte(value) == 169) then
			
				sStringChk = IM_GetData(sName,"realm");

			elseif(string.byte(value) == 248) then
			
				sStringChk = IM_GetData(sName,"level");
				
			elseif(string.byte(value) == 165) then
			
				sStringChk = IM_GetData(sName,"st");
				
			elseif(string.byte(value) == 215) then
			
				sStringChk = IM_GetData(sName,"player");
				
			elseif(string.byte(value) == 177) then
			
				sStringChk = IM_GetData(sName,"vendor");
				
			end
			
			
			if(sStringChk) then
			
				sStoreStr = sStoreStr..value..sStringChk.."";
				
			end
			
		end
		
		
	end
	
	
	--now add the updated
	--convert our sSlotNum to a character duh
	sStoreStr = sStoreStr..string.char(sSlotNum)..sValue.."";
	
	--now update it
	IMDB[IM_Realm][sName] = sStoreStr;
	
	

end



---------------------------------------------------
-- IM_AddData
-- Adds Data to an item array
-----------------------------------------------
function IM_AddData(sName, SDelimiter, sValue)
local sSlotNum;

	--for some reason the byte num from sOpt always comes out 194
	--so to compensate for this we have to assign the ascii byte ourselves
	--""= 187; -- item
	--""= 186; -- quality
	--""= 247; -- price
	--""= 162; -- wl
	--""= 163; -- wt
	--""= 167; -- ts
	--""= 183; -- at
	--""= 165; -- st
	--""= 169; -- realm
	--""= 248; -- level
	--""= 215; -- player
	--""= 177; -- vendor
	
	--make sure we have a item name
	if(not sName) then return nil; end

	--make sure the request being passed is a string
	if(type(sName) ~= "string") then return nil; end
	
	--make sure we have an SDelimiter
	if(not SDelimiter) then return nil; end
	
	--make sure the SDelimiter being passed is a string
	if(type(SDelimiter) ~= "string") then return nil; end
	
	--make sure we have an sValue
	if(not sValue) then return nil; end
	
	--make sure we have a database to work with
	if(not IMDB) then return nil; end
	
	--make sure we have a realm name
	if(not IM_Realm) then return nil; end
	
	--make sure we have the database with the server
	if(not IMDB[IM_Realm]) then return nil; end
	

	--convert to lowercase just in case
	SDelimiter = string.lower(SDelimiter);
	
	if(SDelimiter == "item") then sSlotNum = 187;
	elseif(SDelimiter == "quality") then sSlotNum = 186;
	elseif(SDelimiter == "price") then sSlotNum = 247;
	elseif(SDelimiter == "wl") then sSlotNum = 162;
	elseif(SDelimiter == "wt") then sSlotNum = 163;
	elseif(SDelimiter == "ts") then sSlotNum = 167;
	elseif(SDelimiter == "at") then sSlotNum = 183;
	elseif(SDelimiter == "realm") then sSlotNum = 169;
	elseif(SDelimiter == "level") then sSlotNum = 248;
	elseif(SDelimiter == "player") then sSlotNum = 215;
	elseif(SDelimiter == "vendor") then sSlotNum = 177;
	elseif(SDelimiter == "st") then sSlotNum = 165; end


	--make sure we have information to continue
	if(not sSlotNum or not sName or not sValue) then
		return nil;
	end
	
	--don't create empty named items
	if(sName == "" or sName == " ") then
		return nil;
	end

	--If the item doesn't exist then create it
	if(not IMDB[IM_Realm][sName]) then
		IMDB[IM_Realm][sName] = "";
	end


	sStoreStr = ""; --reset
	sStoreStr = IMDB[IM_Realm][sName]; --store the information there (which should be nothing)
	
	--If the item we want to edit already exists then update it rather then adding
	if( IM_GetData(sName, SDelimiter ) ) then
	
		--then set it
		IM_SetData(sName, SDelimiter, sValue);
		

	else--add it otherwise
	
		sStoreStr = sStoreStr..string.char(sSlotNum)..sValue.."";

		--now update it
		IMDB[IM_Realm][sName] = sStoreStr;
	
	end
	
	

end



--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------